@@ -2,7 +2,7 @@
 #![allow(unused_imports)]
 
 extern crate test;
-#[macro_use(s)]
+#[macro_use(s, azip)]
 extern crate ndarray;
 
 use ndarray::{
@@ -11,6 +11,7 @@ use ndarray::{
     Ix,
     Array1,
     Array2,
+    Zip,
 };
 use ndarray::{arr0, arr1, arr2};
 
@@ -277,6 +278,40 @@ fn add_2d_regular(bench: &mut test::Bencher)
     });
 }
 
+#[bench]
+fn add_2d_zip(bench: &mut test::Bencher)
+{
+    let mut a = Array::<i32, _>::zeros((64, 64));
+    let b = Array::<i32, _>::zeros((64, 64));
+    bench.iter(|| {
+        Zip::from(&mut a).and(&b).apply(|a, &b| *a += b);
+    });
+}
+
+#[bench]
+fn add_2d_alloc(bench: &mut test::Bencher)
+{
+    let a = Array::<i32, _>::zeros((64, 64));
+    let b = Array::<i32, _>::zeros((64, 64));
+    bench.iter(|| {
+        &a + &b
+    });
+}
+
+#[bench]
+fn add_2d_zip_alloc(bench: &mut test::Bencher)
+{
+    let a = Array::<i32, _>::zeros((64, 64));
+    let b = Array::<i32, _>::zeros((64, 64));
+    bench.iter(|| {
+        unsafe {
+            let mut c = Array::uninitialized(a.dim());
+            azip!(a, b, mut c in { *c = a + b });
+            c
+        }
+    });
+}
+
 #[bench]
 fn add_2d_assign_ops(bench: &mut test::Bencher)
 {
@@ -302,6 +337,17 @@ fn add_2d_cutout(bench: &mut test::Bencher)
     });
 }
 
+#[bench]
+fn add_2d_zip_cutout(bench: &mut test::Bencher)
+{
+    let mut a = Array::<i32, _>::zeros((66, 66));
+    let mut acut = a.slice_mut(s![1..-1, 1..-1]);
+    let b = Array::<i32, _>::zeros((64, 64));
+    bench.iter(|| {
+        Zip::from(&mut acut).and(&b).apply(|a, &b| *a += b);
+    });
+}
+
 #[bench]
 fn add_2d_broadcast_1_to_2(bench: &mut test::Bencher)
 {
@@ -392,6 +438,17 @@ fn add_2d_strided(bench: &mut test::Bencher)
     });
 }
 
+#[bench]
+fn add_2d_zip_strided(bench: &mut test::Bencher)
+{
+    let mut a = Array::<i32, _>::zeros((64, 128));
+    let mut a = a.slice_mut(s![.., ..;2]);
+    let b = Array::<i32, _>::zeros((64, 64));
+    bench.iter(|| {
+        Zip::from(&mut a).and(&b).apply(|a, &b| *a += b);
+    });
+}
+
 #[bench]
 fn add_2d_transposed(bench: &mut test::Bencher)
 {
@@ -404,6 +461,17 @@ fn add_2d_transposed(bench: &mut test::Bencher)
     });
 }
 
+#[bench]
+fn add_2d_zip_transposed(bench: &mut test::Bencher)
+{
+    let mut a = Array::<i32, _>::zeros((64, 64));
+    a.swap_axes(0, 1);
+    let b = Array::<i32, _>::zeros((64, 64));
+    bench.iter(|| {
+        Zip::from(&mut a).and(&b).apply(|a, &b| *a += b);
+    });
+}
+
 #[bench]
 fn add_2d_f32_regular(bench: &mut test::Bencher)
 {
